@@ -46,7 +46,6 @@
 #include <KStandardGuiItem>
 #include <KToggleAction>
 #include <KXMLGUIFactory>
-#include <kwidgetsaddons_version.h>
 
 #include <QAction>
 #include <QCursor>
@@ -105,6 +104,7 @@ Part::Part(QWidget *parentWidget, QObject *parent, const KPluginMetaData &metaDa
     // m_vlayout is needed for later insertion of QMessageWidget
     QWidget *mainWidget = new QWidget;
     m_vlayout = new QVBoxLayout;
+    m_vlayout->setSpacing(0);
     m_model = new ArchiveModel(pathName, this);
     m_filterModel = new ArchiveSortFilterModel(this);
     m_splitter = new QSplitter(Qt::Horizontal, parentWidget);
@@ -116,6 +116,7 @@ Part::Part(QWidget *parentWidget, QObject *parent, const KPluginMetaData &metaDa
     m_commentView->setReadOnly(true);
     m_commentView->setFont(QFontDatabase::systemFont(QFontDatabase::FixedFont));
     m_commentBox = new QGroupBox(i18n("Comment"));
+    m_commentBox->setFlat(true);
     m_commentBox->hide();
     QVBoxLayout *vbox = new QVBoxLayout;
     vbox->addWidget(m_commentView);
@@ -123,6 +124,9 @@ Part::Part(QWidget *parentWidget, QObject *parent, const KPluginMetaData &metaDa
 
     m_messageWidget = new KMessageWidget(parentWidget);
     m_messageWidget->setWordWrap(true);
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    m_messageWidget->setPosition(KMessageWidget::Header);
+#endif
     m_messageWidget->hide();
 
     m_commentMsgWidget = new KMessageWidget();
@@ -700,7 +704,7 @@ void Part::updateQuickExtractMenu(QAction *extractAction)
         menu->removeAction(menu->actions().constLast());
     }
 
-    const KConfigGroup conf(KSharedConfig::openConfig(), "ExtractDialog");
+    const KConfigGroup conf(KSharedConfig::openConfig(), QStringLiteral("ExtractDialog"));
     const QStringList dirHistory = conf.readPathEntry("DirHistory", QStringList());
 
     for (int i = 0; i < qMin(10, dirHistory.size()); ++i) {
@@ -875,23 +879,14 @@ bool Part::isLocalFileValid()
 bool Part::confirmAndDelete(const QString &targetFile)
 {
     QFileInfo targetInfo(targetFile);
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
     const auto buttonCode = KMessageBox::warningTwoActions(
         widget(),
-#else
-    const auto buttonCode =
-        KMessageBox::warningYesNo(widget(),
-#endif
         xi18nc("@info", "The archive <filename>%1</filename> already exists. Do you wish to overwrite it?", targetInfo.fileName()),
         i18nc("@title:window", "File Exists"),
         KStandardGuiItem::overwrite(),
         KStandardGuiItem::cancel());
 
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
     if (buttonCode != KMessageBox::PrimaryAction || !targetInfo.isWritable()) {
-#else
-    if (buttonCode != KMessageBox::Yes || !targetInfo.isWritable()) {
-#endif
         return false;
     }
 
@@ -1148,20 +1143,12 @@ void Part::slotWatchedFileModified(const QString& file)
         prettyFilename = relPath + QLatin1Char('/') + file.section(QLatin1Char('/'), -1);
     }
 
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
     if (KMessageBox::questionTwoActions(widget(),
-#else
-    if (KMessageBox::questionYesNo(widget(),
-#endif
                                         xi18n("The file <filename>%1</filename> was modified. Do you want to update the archive?", prettyFilename),
                                         i18nc("@title:window", "File Modified"),
                                         KGuiItem(i18nc("@action:button", "Update"), QStringLiteral("view-refresh")),
                                         KGuiItem(i18nc("@action:button", "Ignore"), QStringLiteral("dialog-cancel")))
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
         == KMessageBox::PrimaryAction) {
-#else
-        == KMessageBox::Yes) {
-#endif
         QStringList list = QStringList() << file;
 
         qCDebug(ARK) << "Updating file" << file << "with path" << relPath;
@@ -1681,11 +1668,7 @@ void Part::slotDeleteFilesDone(KJob* job)
 void Part::slotDeleteFiles()
 {
     const int selectionsCount = m_view->selectionModel()->selectedRows().count();
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
     const auto reallyDelete = KMessageBox::questionTwoActions(widget(),
-#else
-    const auto reallyDelete = KMessageBox::questionYesNo(widget(),
-#endif
                                                               i18ncp("@info",
                                                                      "Deleting this file is not undoable. Are you sure you want to do this?",
                                                                      "Deleting these files is not undoable. Are you sure you want to do this?",
@@ -1696,11 +1679,7 @@ void Part::slotDeleteFiles()
                                                               QString(),
                                                               KMessageBox::Dangerous | KMessageBox::Notify);
 
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
     if (reallyDelete == KMessageBox::SecondaryAction) {
-#else
-    if (reallyDelete == KMessageBox::No) {
-#endif
         return;
     }
 
@@ -1772,7 +1751,12 @@ void Part::slotShowContextMenu()
     }
 
     QMenu *popup = static_cast<QMenu *>(factory()->container(QStringLiteral("context_menu"), this));
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     if (KHamburgerMenu * const hamburgerMenu = static_cast<KHamburgerMenu *>(actionCollection()->action(QLatin1String(KStandardAction::name(KStandardAction::HamburgerMenu))))) {
+#else
+    if (KHamburgerMenu *const hamburgerMenu =
+            static_cast<KHamburgerMenu *>(actionCollection()->action(KStandardAction::name(KStandardAction::HamburgerMenu)))) {
+#endif
         hamburgerMenu->insertIntoMenuBefore(popup, popup->actions().constFirst());
     }
     popup->popup(QCursor::pos());
@@ -1828,3 +1812,4 @@ void Part::displayMsgWidget(KMessageWidget::MessageType type, const QString& msg
 
 } // namespace Ark
 
+#include "moc_part.cpp"
