@@ -1,5 +1,6 @@
 /*
     SPDX-FileCopyrightText: 2017 Ragnar Thomsen <rthomsen6@gmail.com>
+    SPDX-FileCopyrightText: 2023 Ilya Pominov <ipominov@astralinux.ru>
 
     SPDX-License-Identifier: BSD-2-Clause
 */
@@ -18,6 +19,7 @@
 #include <QDir>
 #include <QDirIterator>
 #include <QFile>
+#include <QFileInfo>
 #include <qplatformdefs.h>
 #include <QThread>
 
@@ -32,6 +34,177 @@ using deleter_from_fn = std::integral_constant<decltype(fn), fn>;
 template <typename T, auto fn>
 using ark_unique_ptr = std::unique_ptr<T, deleter_from_fn<fn>>;
 
+class ZipSource
+{
+public:
+    ZipSource(const QString &fileName)
+    {
+        const auto &file = m_files.emplace_back(std::make_unique<QFile>(fileName));
+        m_length = static_cast<zip_uint64_t>(file->size());
+        m_multiVolumeName = fileName;
+        zip_error_init(&m_error);
+
+        // Multi-volume zip's are named name.zip.001, try find other parts
+        if (fileName.endsWith(QStringLiteral(".zip.001"), Qt::CaseInsensitive)) {
+            m_multiVolumeName.resize(m_multiVolumeName.size() - 4);
+            auto fNameSize = fileName.size();
+            for (int i = 2; i <= 999; ++i) {
+                auto partFileName = fileName;
+                partFileName.replace(fNameSize - 3, 3, QStringLiteral("%1").arg(i, 3, 10, QLatin1Char('0')));
+                if (!QFileInfo::exists(partFileName)) {
+                    break;
+                }
+                const auto &part = m_files.emplace_back(std::make_unique<QFile>(partFileName));
+                m_length += static_cast<zip_uint64_t>(part->size());
+            }
+        }
+    }
+
+    int numberOfVolumes() const
+    {
+        return static_cast<int>(m_files.size());
+    }
+
+    QString multiVolumeName() const
+    {
+        return m_multiVolumeName;
+    }
+
+    zip_int64_t stat(zip_stat_t *info)
+    {
+        zip_stat_init(info);
+        if (!info) {
+            zip_error_set(&m_error, ZIP_ER_ZLIB, 0);
+            return -1;
+        }
+
+        info->size = m_length;
+        info->valid = ZIP_STAT_SIZE;
+        return sizeof(struct zip_stat);
+    }
+
+    zip_int64_t seek(void *data, zip_uint64_t len)
+    {
+        zip_int64_t newOffset = zip_source_seek_compute_offset(m_offset, m_length, data, len, &m_error);
+        if (newOffset < 0) {
+            zip_error_set(&m_error, ZIP_ER_SEEK, 0);
+            return -1;
+        }
+
+        m_offset = static_cast<zip_uint64_t>(newOffset);
+        return 0;
+    }
+
+    zip_int64_t read(void *data, zip_uint64_t len)
+    {
+        if (len == 0 || m_offset >= m_length) {
+            return 0;
+        }
+
+        zip_int64_t ret = 0;
+        zip_uint64_t offset = m_offset;
+        for (auto &file : m_files) {
+            const auto length = static_cast<zip_uint64_t>(file->size());
+            if (offset >= length) {
+                offset -= length;
+                continue;
+            }
+
+            if (!file->isOpen() && !file->open(QIODevice::ReadOnly)) {
+                qCDebug(ARK) << "ZipSource error: Can't open" << file->fileName();
+                break;
+            }
+
+            const auto available = (length - offset) < len ? length - offset : len;
+            if (!file->seek(static_cast<qint64>(offset))) {
+                qCDebug(ARK) << "ZipSource error: Can't seek to" << available << "in file" << file->fileName();
+                break;
+            }
+
+            const auto readed = file->read(reinterpret_cast<char *>(data), static_cast<qint64>(available));
+            if (static_cast<zip_uint64_t>(readed) != available) {
+                qCDebug(ARK) << "ZipSource error: Read" << readed << "bytes instead" << available << "in file" << file->fileName();
+                break;
+            }
+
+            ret += static_cast<zip_int64_t>(available);
+            m_offset += available;
+            offset = 0;
+            len -= available;
+            data = reinterpret_cast<char *>(data) + available;
+            if (len == 0) {
+                return ret;
+            }
+        }
+
+        zip_error_set(&m_error, ZIP_ER_READ, 0);
+        return -1;
+    }
+
+    // Commands should return -1 on error. ZIP_SOURCE_ERROR will be called to retrieve the error code.
+    // On success, commands return 0, unless specified otherwise in the description above.
+    // See https://libzip.org/documentation/zip_source_function_create.html for more details.
+    static zip_int64_t callbackFn(void *userdata, void *data, zip_uint64_t len, zip_source_cmd_t cmd)
+    {
+        auto source = reinterpret_cast<ZipSource *>(userdata);
+        switch(cmd)
+        {
+            case ZIP_SOURCE_OPEN:
+                return 0;
+            case ZIP_SOURCE_READ:
+                return source->read(data, len);
+            case ZIP_SOURCE_CLOSE:
+                return 0;
+            case ZIP_SOURCE_STAT:
+                return source->stat(reinterpret_cast<zip_stat_t *>(data));
+            case ZIP_SOURCE_ERROR:
+                return zip_error_to_data(&source->m_error, data, len);
+            case ZIP_SOURCE_FREE:
+                return 0;
+            case ZIP_SOURCE_SEEK:
+                return source->seek(data, len);
+            case ZIP_SOURCE_TELL:
+                return static_cast<zip_int64_t>(source->m_offset);
+            case ZIP_SOURCE_SUPPORTS:
+                return ZIP_SOURCE_SUPPORTS_SEEKABLE;
+            default:
+                zip_error_set(&source->m_error, ZIP_ER_INVAL, 0);
+                break;
+        }
+        return -1;
+    }
+
+    static ark_unique_ptr<zip_t, zip_discard> create(LibzipPlugin *plugin, ZipSource &zipSource, int zipOpenFlags)
+    {
+        zip_error_t err;
+        zip_error_init(&err);
+        ark_unique_ptr<zip_t, zip_discard> archive;
+        if (plugin->isMultiVolume()) {
+            auto source = zip_source_function_create(&ZipSource::callbackFn, &zipSource, nullptr);
+            archive.reset(zip_open_from_source(source, zipOpenFlags, &err));
+            if (!archive) {
+                zip_source_free(source);
+            }
+        } else {
+            int errcode = 0;
+            archive.reset(zip_open(QFile::encodeName(plugin->filename()).constData(), zipOpenFlags, &errcode));
+            zip_error_init_with_code(&err, errcode);
+        }
+        if (!archive) {
+            qCCritical(ARK) << "Failed to open archive. Code:" << zip_error_code_zip(&err);
+            Q_EMIT plugin->error(xi18n("Failed to open archive: %1", QString::fromUtf8(zip_error_strerror(&err))));
+        }
+        return archive;
+    }
+
+private:
+    std::vector<std::unique_ptr<QFile> > m_files;
+    QString m_multiVolumeName;
+    zip_error_t m_error;
+    zip_uint64_t m_length = 0;
+    zip_uint64_t m_offset = 0;
+};
+
 void LibzipPlugin::progressCallback(zip_t *, double progress, void *that)
 {
     static_cast<LibzipPlugin *>(that)->emitProgress(progress);
@@ -48,8 +221,15 @@ LibzipPlugin::LibzipPlugin(QObject *parent, const QVariantList & args)
     , m_skipAll(false)
     , m_listAfterAdd(false)
     , m_backslashedZip(false)
+    , m_zipSource(std::make_unique<ZipSource>(filename()))
 {
     qCDebug(ARK) << "Initializing libzip plugin";
+
+    if (m_zipSource->numberOfVolumes() > 1) {
+        m_numberOfVolumes = m_zipSource->numberOfVolumes();
+        setMultiVolume(true);
+        m_multiVolumeName = m_zipSource->multiVolumeName();
+    }
 }
 
 LibzipPlugin::~LibzipPlugin()
@@ -65,15 +245,9 @@ bool LibzipPlugin::list()
     qCDebug(ARK) << "Listing archive contents for:" << QFile::encodeName(filename());
     m_numberOfEntries = 0;
 
-    int errcode = 0;
-    zip_error_t err;
-
     // Open archive.
-    ark_unique_ptr<zip_t, zip_discard> archive { zip_open(QFile::encodeName(filename()).constData(), ZIP_RDONLY, &errcode) };
-    zip_error_init_with_code(&err, errcode);
+    auto archive = ZipSource::create(this, *m_zipSource, ZIP_RDONLY);
     if (!archive) {
-        qCCritical(ARK) << "Failed to open archive. Code:" << errcode;
-        Q_EMIT error(xi18n("Failed to open archive: %1", QString::fromUtf8(zip_error_strerror(&err))));
         return false;
     }
 
@@ -486,14 +660,10 @@ bool LibzipPlugin::addComment(const QString& comment)
 bool LibzipPlugin::testArchive()
 {
     qCDebug(ARK) << "Testing archive";
-    int errcode = 0;
-    zip_error_t err;
 
     // Open archive performing extra consistency checks, free memory using zip_discard as no write oprations needed.
-    ark_unique_ptr<zip_t, zip_discard> archive { zip_open(QFile::encodeName(filename()).constData(), ZIP_CHECKCONS, &errcode) };
-    zip_error_init_with_code(&err, errcode);
-    if (archive == nullptr) {
-        qCCritical(ARK) << "Failed to open archive:" << zip_error_strerror(&err);
+    auto archive = ZipSource::create(this, *m_zipSource, ZIP_RDONLY | ZIP_CHECKCONS);
+    if (!archive) {
         return false;
     }
 
@@ -544,15 +714,9 @@ bool LibzipPlugin::extractFiles(const QVector<Archive::Entry*> &files, const QSt
     const bool extractAll = files.isEmpty();
     const bool removeRootNode = options.isDragAndDropEnabled();
 
-    int errcode = 0;
-    zip_error_t err;
-
     // Open archive, free memory using zip_discard as no write oprations needed.
-    ark_unique_ptr<zip_t, zip_discard> archive { zip_open(QFile::encodeName(filename()).constData(), ZIP_RDONLY, &errcode) };
-    zip_error_init_with_code(&err, errcode);
-    if (archive == nullptr) {
-        qCCritical(ARK) << "Failed to open archive. Code:" << errcode;
-        Q_EMIT error(xi18n("Failed to open archive: %1", QString::fromUtf8(zip_error_strerror(&err))));
+    auto archive = ZipSource::create(this, *m_zipSource, ZIP_RDONLY);
+    if (!archive) {
         return false;
     }
 
@@ -990,4 +1154,15 @@ bool LibzipPlugin::hasBatchExtractionProgress() const
     return true;
 }
 
+bool LibzipPlugin::isReadOnly() const
+{
+    return isMultiVolume() || ReadWriteArchiveInterface::isReadOnly();
+}
+
+QString LibzipPlugin::multiVolumeName() const
+{
+    return m_multiVolumeName.isEmpty() ? filename() : m_multiVolumeName;
+}
+
 #include "libzipplugin.moc"
+#include "moc_libzipplugin.cpp"
